// Copyright (C) 2008
// by all contributors..

// This file is part of SCNSL.

// SCNSL is free software:
// you can redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option)
// any later version.

// SCNSL is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with SCNSL,
// in a file named LICENSE.txt.
// If not, see <http://www.gnu.org/licenses/>.



/// @file
/// A generic packet.


#include <cstring>

#include "Packet_t.hh"
#include "Task_if_t.hh"
#include "TaskProxy_if_t.hh"

using Scnsl::Core::Packet_t;



// ////////////////////////////////////////////////////////////////
// Regular design pattern.
// ////////////////////////////////////////////////////////////////

Packet_t::Packet_t()
    throw ():
    // Payload fields.
    _buffer( NULL ),
    _buffer_size( 0 ),
    _buffer_capacity( 0 ),
    _packet( NULL ),
    // Header fields.
    _is_ack( false ),
    _is_ack_required( false ),
    _sequence_number( 0 ),
    _label( 0 ),
    _header_size( 0 ),
    _sourceTaskproxy( NULL ),
    _destinationTask( NULL ),
    _sourceNode( NULL ),
    _destinationNode( NULL ),
    // Simulation fields.
    _id( 0 ),
    _channel( NULL ),
    // To be removed.
    _is_valid( true ),
    _sourceProperties()
{
    // Nothing to do.
}


Packet_t::Packet_t( const Packet_t & p )
    throw ( std::bad_alloc ):
    // Payload fields.
    _buffer( NULL ),
    _buffer_size( p._buffer_size ),
    _buffer_capacity( p._buffer_capacity ),
    _packet( NULL ),
    // Header fields.
    _is_ack( p._is_ack ),
    _is_ack_required( p._is_ack_required ),
    _sequence_number( p._sequence_number ),
    _label( p._label ),
    _header_size( p._header_size ),
    _sourceTaskproxy( p._sourceTaskproxy ),
    _destinationTask( p._destinationTask ),
    _sourceNode( p._sourceNode ),
    _destinationNode( p._destinationNode ),
    // Simulation fields.
    _id( p._id ),
    _channel( p._channel ),
    // To be removed.
    _is_valid( p._is_valid ),
    _sourceProperties(p._sourceProperties)
{
    if ( p._packet != NULL )
    {
        _packet = new Packet_t( * p._packet );
    }
    else
    {
        if ( _buffer_capacity == 0 ) return;
        _buffer = new byte_t[ _buffer_capacity ];
        memcpy( _buffer, p._buffer, _buffer_capacity );
    }
}


Packet_t::~Packet_t()
    throw ()
{
    delete [] _buffer;
    delete _packet;
}

Packet_t & Packet_t::operator = ( const Packet_t & p )
    throw ( std::bad_alloc )
{
    // Packet fields.
    // Exception neutral.
    if ( p._packet != NULL )
    {
        if ( _packet == NULL )
        {
            _packet = new Packet_t( * p._packet );
        }
        else
        {
            *_packet = *p._packet;
        }
        delete [] _buffer;
        _buffer = NULL;
        _buffer_size = 0;
        _buffer_capacity = 0;
    }
    else
    {
        if ( _buffer_capacity < to_bytes(p._buffer_size) )
        {
            byte_t * b = new byte_t[ to_bytes(p._buffer_size) ];
            delete [] _buffer;
            _buffer = b;
            _buffer_capacity = to_bytes(p._buffer_size);
        }

        memcpy( _buffer, p._buffer, to_bytes(p._buffer_size) );
        _buffer_size = p._buffer_size;

        delete _packet;
        _packet = NULL;
    }

    // Header fields.
    _is_ack = p._is_ack;
    _is_ack_required = p._is_ack_required;
    _sequence_number = p._sequence_number;
    _label = p._label;
    _header_size = p._header_size;
    _sourceTaskproxy = p._sourceTaskproxy;
    _destinationTask = p._destinationTask;
    _sourceNode = p._sourceNode;
    _destinationNode = p._destinationNode;
    // Simulation fields.
    _id = p._id;
    _channel = p._channel;
    // To be removed.
    _is_valid = p._is_valid;
    _sourceProperties = p._sourceProperties;

    return *this;
}


// ////////////////////////////////////////////////////////////////
// Payload related methods.
// ////////////////////////////////////////////////////////////////

bool Packet_t::hasBufferPayload()
    const throw ()
{
    return ( _packet == NULL );
}


Packet_t::size_t Packet_t::getPayloadSize()
    const throw ()
{
    if ( _packet == NULL )
        return _buffer_size;

    return _packet->getSize();
}



Packet_t::size_t Packet_t::getPayloadSizeInBytes()
    const throw ()
{
    return to_bytes( getPayloadSize() );
}



void Packet_t::setPayload( const byte_t * b, const size_t s )
    throw ( std::bad_alloc )
{
    setPayloadInBytes( b, to_bytes( s ) );
}

void Packet_t::setPayloadInBytes( const byte_t * b, const size_t s )
    throw ( std::bad_alloc )
{
    if ( _buffer_capacity < s )
    {
        byte_t * buf = new byte_t[ s ];
        delete [] _buffer;
        _buffer = buf;
        _buffer_capacity = s;
    }

    memcpy( _buffer, b, s );
    _buffer_size = to_bits( s );

    if ( _packet != NULL )
    {
        delete _packet;
        _packet = NULL;
    }
}

void Packet_t::setPayload( const Packet_t & p )
    throw ( std::bad_alloc )
{
    if ( _packet == NULL )
    {
        _packet = new Packet_t( p );
        delete [] _buffer;
        _buffer = NULL;
        _buffer_size = 0;
        _buffer_capacity = 0;

        return;
    }

    * _packet = p;
}


void Packet_t::getPayload( byte_t const * & b )
    const throw ()
{
    assert( _packet == NULL );
    b = _buffer;
}

void Packet_t::getPayload( byte_t * & b )
    throw ()
{
    assert( _packet == NULL );
    b = _buffer;
}

void Packet_t::getPayload( Packet_t & p )
    const throw ()
{
    assert( _packet != NULL );
    p = *_packet;
}

Packet_t* Packet_t::getInnerPayload( )
          const throw ()
{
    assert( _packet != NULL );
    return _packet;
          
}   


// ////////////////////////////////////////////////////////////////
// Header methods.
// ////////////////////////////////////////////////////////////////

Packet_t::size_t Packet_t::getHeaderSize()
    const throw ()
{
    return _header_size;
}


Packet_t::size_t Packet_t::getHeaderSizeInBytes()
    const throw ()
{
    return to_bytes( _header_size );
}

void Packet_t::setHeaderSize( const size_t s )
    throw ()
{
    _header_size = s;
}

void Packet_t::setHeaderSizeInBytes( const size_t s )
    throw ()
{
    _header_size = to_bits( s );
}

bool Packet_t::isAck()
    const throw ()
{
    return _is_ack;
}


void Packet_t::setAck( const bool ack )
    throw ()
{
    _is_ack = ack;
}


bool Packet_t::isAckRequired()
    const throw ()
{
    return _is_ack_required;
}


void Packet_t::setAckRequired( const bool ack )
    throw ()
{
    _is_ack_required = ack;
}


Packet_t::counter_t Packet_t::getSequenceNumber()
    const throw ()
{
    return _sequence_number;
}


void Packet_t::setSequenceNumber( const counter_t seqno )
    throw ()
{
    _sequence_number = seqno;
}


Packet_t::label_t Packet_t::getLabel()
    const throw ()
{
    return _label;
}

void Packet_t::setLabel( const label_t l )
    throw ()
{
    _label = l;
}


void Packet_t::setSourceTaskProxy( const TaskProxy_if_t * tp )
    throw ()
{
    _sourceTaskproxy = tp;
}

const Scnsl::Core::TaskProxy_if_t * Packet_t::getSourceTaskProxy()
    const throw ()
{
    return _sourceTaskproxy;
}

void Packet_t::setDestinationTask( const Task_if_t * t )
    throw ()
{
    _destinationTask = t;
}

const Scnsl::Core::Task_if_t * Packet_t::getDestinationTask()
    const throw ()
{
    return _destinationTask;
}


void Packet_t::setSourceNode( const Node_t * n )
    throw ()
{
    _sourceNode = n;
}

const Scnsl::Core::Node_t * Packet_t::getSourceNode()
    const throw ()
{
    if ( _sourceNode != NULL ) return _sourceNode;
    if ( _sourceTaskproxy != NULL ) return _sourceTaskproxy->getTask()->getNode();
    return NULL;
}

void Packet_t::setDestinationNode( const Node_t * n )
    throw ()
{
    _destinationNode = n;
}

const Scnsl::Core::Node_t * Packet_t::getDestinationNode()
    const throw ()
{
    if ( _destinationNode != NULL ) return _destinationNode;
    if ( _destinationTask != NULL ) return _destinationTask->getNode();
    return NULL;
}





Packet_t::size_t Packet_t::getSize()
    const throw ()
{
    return getPayloadSize() + getHeaderSize();
}

Packet_t::size_t Packet_t::getSizeInBytes()
    const throw ()
{
    return getPayloadSizeInBytes() + getHeaderSizeInBytes();
}


void Packet_t::setSourceProperties(node_properties_t & sp)
{
     _sourceProperties = sp;
}


const Packet_t::node_properties_t & Packet_t::getSourceProperties()
     const
{
     return _sourceProperties;
}

// ////////////////////////////////////////////////////////////////
// Simulation methods.
// ////////////////////////////////////////////////////////////////


Packet_t::packet_id_t Packet_t::getId()
    const throw ()
{
    return _id;
}

void Packet_t::setId( const packet_id_t id )
    throw ()
{
    _id = id;
}

void Packet_t::setChannel( const Channel_if_t * ch )
    throw ()
{
    _channel = ch;
}

const Scnsl::Core::Channel_if_t * Packet_t::getChannel()
    const throw ()
{
    return _channel;
}



// ////////////////////////////////////////////////////////////////
// Support methods.
// ////////////////////////////////////////////////////////////////



bool Packet_t::operator == ( const Packet_t & p )
    const throw ()
{
    // We do not consider mixed payload cases.
    if ( _packet != NULL && p._packet == NULL ) return false;
    if ( _packet == NULL && p._packet != NULL ) return false;

    // Header fields.
    if ( _is_ack != p._is_ack
         || _is_ack_required !=  p._is_ack_required
         || _sequence_number != p._sequence_number
         || _label != p._label
         || _header_size != p._header_size
         || _sourceTaskproxy != p._sourceTaskproxy
         || _destinationTask != p._destinationTask
         || _sourceNode != p._sourceNode
         || _destinationNode != p._destinationNode )
        return false;

    // Simulation fields.
    // We do not consider simulation fields.
    // if ( _id != p._id
    //      || _channel != p._channel )
    //     return false;

    // To be removed fields.
    // We do not consider these fields.
    // if ( _is_valid != p._is_valid
    //      || _printer != p._printer )
    //     return false;


    //
    // Finally, check packet/buffer contents:
    //

    if ( _packet != NULL ) return (*_packet == *p._packet);

    if ( _buffer_size != p._buffer_size ) return false;
    return ( memcmp( _buffer, p._buffer, to_bytes(_buffer_size) ) ) == 0;
}



// ////////////////////////////////////////////////////////////////
// To be removed methods.
// ////////////////////////////////////////////////////////////////


bool Packet_t::isValid()
    const throw ()
{
    return _is_valid;
}

void Packet_t::setValidity( const bool v )
    throw ()
{
    _is_valid = v;
}



// ////////////////////////////////////////////////////////////////
// Static support methods.
// ////////////////////////////////////////////////////////////////


Packet_t::packet_id_t Packet_t::get_new_packet_id()
    throw ()
{
    static packet_id_t id = 0;
    return id++;
}



Packet_t::size_t Packet_t::to_bytes( const size_t s )
    throw ()
{
    return (s / 8) + ( s % 8 ? 1 : 0);
}

Packet_t::size_t Packet_t::to_bits( const size_t s )
    throw ()
{
    return s * 8;
}

